use modules::EventLoop;
use toml::value::Table;

use std::sync::Arc;
use std::sync::mpsc::Receiver;
use transformable_channels::mpsc::ExtSender;

use regex;
use regex::Regex;

use event::{Event, Envelope};
use rand;

pub struct RandomModule {
    initial_configuration: Table,
}

impl RandomModule {
    pub fn new (_: &Table, configuration: &Table) -> Box<EventLoop> {
        Box::new(RandomModule {
            initial_configuration: configuration.clone()
        })
    }

    fn pattern_from_config (configuration: &Table) -> Regex {
        configuration.get("pattern")
                     .and_then(|value| value.as_str())
                     .map(String::from)
                     .or_else(|| configuration.get("prefix")
                                               .and_then(|value| value.as_str())
                                               .map(|value| format!("^{}", regex::escape(value))))
                     .and_then(|value| Regex::new(&value).ok())
                     .expect("Invalid value for pattern")
    }

    fn responses_from_config (configuration: &Table) -> Vec<String> {
        configuration.get("responses")
                     .and_then(|value| value.as_array())
                     .map(|value| value.to_vec())
                     .unwrap_or(vec![])
                     .into_iter()
                     .map(|value| { String::from(value.as_str().unwrap()) })
                     .collect()
    }
}

impl EventLoop for RandomModule {
    fn run (&self, _: Box<ExtSender<Event>>, receiver: Receiver<Arc<Envelope>>) {
        let mut rng = rand::thread_rng();
        let mut pattern = RandomModule::pattern_from_config(&self.initial_configuration);
        let mut responses = RandomModule::responses_from_config(&self.initial_configuration);

        loop {
            match receiver.recv() {
                Ok(envelope) => {
                    match envelope.event {
                        Event::Configure { ref configuration } => {
                            pattern = RandomModule::pattern_from_config(configuration);
                            responses = RandomModule::responses_from_config(configuration);
                        },
                        Event::Message { ref message } => {
                            debug!("Received message from module {:?}... {:?}", envelope.from, message.content);
                            if let Some(captures) = pattern.captures(&message.content) {
                                let mut response = String::new();
                                captures.expand(&rand::sample(&mut rng, &responses, 1)[0], &mut response);
                                message.reply(&response);
                            }
                        },
                        _ => {}
                    }
                }
                Err(error) => { error!("Error {:?}", error) }
            }
        }
    }
}
